@@ -19,6 +19,7 @@ namespace glmmtmb{
     if(!give_log) return exp(logres);
     else return logres;
   }
+	
   template<class Type>
   Type dgenpois(Type y, Type theta, Type lambda, int give_log=0)
   {
@@ -32,6 +33,34 @@ namespace glmmtmb{
     if(!give_log) return exp(logres);
     else return logres;
   }
+	
+  /* Simulate from generalized poisson distribution */
+  template<class Type>
+  Type rgenpois(Type theta, Type lambda) {
+    // Copied from R function HMMpa::rgenpois
+    Type ans = Type(0);
+    Type random_number = runif(Type(0), Type(1));
+    Type kum = dgenpois(Type(0), theta, lambda);
+    while (random_number > kum) {
+      ans = ans + Type(1);
+      kum += dgenpois(ans, theta, lambda);
+    }
+    return ans;
+  }
+	
+  /* Simulate from zero-truncated generalized poisson distribution */
+  template<class Type>
+  Type rtruncated_genpois(Type theta, Type lambda) {
+    int nloop = 10000;
+    int counter = 0;
+    Type ans = rgenpois(theta, lambda);
+    while(ans < Type(1) && counter < nloop) {
+      ans = rgenpois(theta, lambda);
+      counter++;
+    }
+    if(ans < 1.) warning("Zeros in simulation of zero-truncated data. Possibly due to low estimated mean.");
+    return ans;
+  }
 
   template<class Type>
   bool isNA(Type x){
@@ -119,18 +148,29 @@ namespace glmmtmb{
     return ans.getDeriv()[0];
   }
 
+  /* Simulate from zero-truncated Conway-Maxwell-Poisson distribution */
+  template<class Type>
+  Type rtruncated_compois2(Type mean, Type nu) {
+    int nloop = 10000;
+    int counter = 0;
+    Type ans = rcompois2(mean, nu);
+    while(ans < 1. && counter < nloop) {
+      ans = rcompois2(mean, nu);
+      counter++;
+    }
+    if(ans < 1.) warning("Zeros in simulation of zero-truncated data. Possibly due to low estimated mean.");
+    return ans;
+  }
+
   /* Simulate from tweedie distribution */
   template<class Type>
-  Type rtweedie(Type mu_, Type phi_, Type p_) {
-    double mu = asDouble(mu_);
-    double phi = asDouble(phi_);
-    double p = asDouble(p_);
+  Type rtweedie(Type mu, Type phi, Type p) {
     // Copied from R function tweedie::rtweedie
-    double lambda = pow(mu, 2. - p) / (phi * (2. - p));
-    double alpha  = (2. - p) / (1. - p);
-    double gam = phi * (p - 1.) * pow(mu, p - 1.);
-    int N = (int) rpois(lambda);
-    double ans = rgamma(N, -alpha /* shape */, gam /* scale */).sum();
+    Type lambda = pow(mu, 2. - p) / (phi * (2. - p));
+    Type alpha  = (2. - p) / (1. - p);
+    Type gam = phi * (p - 1.) * pow(mu, p - 1.);
+    int N = (int) asDouble(rpois(lambda));
+    Type ans = rgamma(N, -alpha /* shape */, gam /* scale */).sum();
     return ans;
   }
 }
@@ -164,6 +204,8 @@ enum valid_family {
   truncated_poisson_family =401,
   genpois_family =402,
   compois_family =403,
+  truncated_genpois_family =404,
+  truncated_compois_family =405,
   nbinom1_family =500,
   nbinom2_family =501,
   truncated_nbinom1_family =502,
@@ -538,6 +580,8 @@ Type objective_function<Type>::operator() ()
   DATA_VECTOR(size); //only used in binomial
   DATA_VECTOR(weights);
   DATA_VECTOR(offset);
+  DATA_VECTOR(zioffset);
+  DATA_VECTOR(doffset);
 
   // Define covariance structure for the conditional model
   DATA_STRUCT(terms, terms_t);
@@ -580,8 +624,8 @@ Type objective_function<Type>::operator() ()
 
   // Linear predictor
   vector<Type> eta = X * beta + Z * b + offset;
-  vector<Type> etazi = Xzi * betazi + Zzi * bzi;
-  vector<Type> etad = Xd * betad;
+  vector<Type> etazi = Xzi * betazi + Zzi * bzi + zioffset;
+  vector<Type> etad = Xd * betad + doffset;
 
   // Apply link
   vector<Type> mu(eta.size());
@@ -703,6 +747,15 @@ Type objective_function<Type>::operator() ()
         s1 = mu(i) / sqrt(phi(i)); //theta
         s2 = Type(1) - Type(1)/sqrt(phi(i)); //lambda
         tmp_loglik = glmmtmb::dgenpois(yobs(i), s1, s2, true);
+        SIMULATE{yobs(i)=glmmtmb::rgenpois(mu(i) / sqrt(phi(i)), Type(1) - Type(1)/sqrt(phi(i)));}
+        break;
+      case truncated_genpois_family:
+        s1 = mu(i) / sqrt(phi(i)); //theta
+        s2 = Type(1) - Type(1)/sqrt(phi(i)); //lambda
+        log_nzprob = logspace_sub(Type(0), -s1);
+        tmp_loglik = glmmtmb::dgenpois(yobs(i), s1, s2, true) - log_nzprob;
+        if( yobs(i) < Type(1) ) tmp_loglik = -INFINITY;
+        SIMULATE{yobs(i)=glmmtmb::rtruncated_genpois(mu(i) / sqrt(phi(i)), Type(1) - Type(1)/sqrt(phi(i)));}
         break;
       case compois_family:
         s1 = mu(i); //mean
@@ -710,6 +763,14 @@ Type objective_function<Type>::operator() ()
         tmp_loglik = dcompois2(yobs(i), s1, s2, true);
         SIMULATE{yobs(i)=rcompois2(mu(i), 1/phi(i));}
         break;
+      case truncated_compois_family:
+        s1 = mu(i); //mean
+        s2 = 1/phi(i); //nu
+        log_nzprob = logspace_sub(Type(0), dcompois2(Type(0), s1, s2, true));
+        tmp_loglik = dcompois2(yobs(i), s1, s2, true) - log_nzprob;
+        if( yobs(i) < Type(1) ) tmp_loglik = -INFINITY;
+        SIMULATE{yobs(i)=glmmtmb::rtruncated_compois2(mu(i), 1/phi(i));}
+        break;
       case tweedie_family:
         s1 = mu(i);  // mean
         s2 = phi(i); // phi
