@@ -74,7 +74,9 @@ test_that("Variance structures", {
 })
 
 test_that("Sleepdata Variance components", {
-    ## TODO: Variance Components ("theta"s)
+    expect_equal(c(unlist(VarCorr(fm3))),
+                 c(cond.Subject = 584.247907378213, cond.Subject.1 = 33.6332741779585),
+                 tolerance=1e-5)
 })
 
 gm0 <<- glmmTMB(cbind(incidence, size-incidence) ~ 1 +      (1|herd),
@@ -176,7 +178,8 @@ test_that("close to lme4 results", {
 context("trickier examples")
 
 data(Owls)
-Owls <<- transform(Owls,
+## is <<- necessary ... ?
+Owls <- transform(Owls,
                    ArrivalTime=scale(ArrivalTime,center=TRUE,scale=FALSE),
                    NCalls= SiblingNegotiation) 
 
